@@ -26,10 +26,10 @@ from __future__ import division
 
 from builtins import str
 from builtins import object
+import datetime
+import logging
 import os
 import sys
-import logging
-import datetime
 
 MIN = 0
 ERROR = 0
@@ -40,17 +40,18 @@ DEBUG = 9
 MAX = 9
 
 _logger = None
+_log_timestamp = False
 
 
 def DupToLoggerLevel(verb):
     u"""Convert duplicity level to the logging module's system, where higher is
-       more severe"""
+        more severe"""
     return MAX - verb + 1
 
 
 def LoggerToDupLevel(verb):
-    u"""Convert logging module level to duplicity's system, where lowere is
-       more severe"""
+    u"""Convert logging module level to duplicity's system, where lower is
+        more severe"""
     return DupToLoggerLevel(verb)
 
 
@@ -67,7 +68,7 @@ def LevelName(level):
         return u"ERROR"
 
 
-def Log(s, verb_level, code=1, extra=None, force_print=False):
+def Log(s, verb_level, code=1, extra=None, force_print=False, transfer_progress=False):
     u"""Write s to stderr if verbosity level low enough"""
     global _logger
     if extra:
@@ -90,7 +91,8 @@ def Log(s, verb_level, code=1, extra=None, force_print=False):
 
     _logger.log(DupToLoggerLevel(verb_level), s,
                 extra={u'levelName': LevelName(verb_level),
-                       u'controlLine': controlLine})
+                       u'controlLine': controlLine,
+                       u'transferProgress': transfer_progress})
 
     if force_print:
         _logger.setLevel(initial_level)
@@ -103,8 +105,8 @@ def Debug(s):
 
 class InfoCode(object):
     u"""Enumeration class to hold info code values.
-       These values should never change, as frontends rely upon them.
-       Don't use 0 or negative numbers."""
+        These values should never change, as frontends rely upon them.
+        Don't use 0 or negative numbers."""
     generic = 1
     progress = 2
     collection_status = 3
@@ -214,7 +216,7 @@ def TransferProgress(progress, eta, changed_bytes, elapsed, speed, stalled):
                                                        )
 
     controlLine = u"%d %d %d %d %d %d" % (changed_bytes, elapsed, progress, eta, speed, stalled)
-    Log(s, NOTICE, InfoCode.upload_progress, controlLine)
+    Log(s, NOTICE, InfoCode.upload_progress, controlLine, transfer_progress=True)
 
 
 def PrintCollectionStatus(col_stats, force_print=False):
@@ -235,8 +237,8 @@ def Notice(s):
 
 class WarningCode(object):
     u"""Enumeration class to hold warning code values.
-       These values should never change, as frontends rely upon them.
-       Don't use 0 or negative numbers."""
+        These values should never change, as frontends rely upon them.
+        Don't use 0 or negative numbers."""
     generic = 1
     orphaned_sig = 2
     unnecessary_sig = 3
@@ -259,9 +261,9 @@ def Warn(s, code=WarningCode.generic, extra=None):
 
 class ErrorCode(object):
     u"""Enumeration class to hold error code values.
-       These values should never change, as frontends rely upon them.
-       Don't use 0 or negative numbers.  This code is returned by duplicity
-       to indicate which error occurred via both exit code and log."""
+        These values should never change, as frontends rely upon them.
+        Don't use 0 or negative numbers.  This code is returned by duplicity
+        to indicate which error occurred via both exit code and log."""
     generic = 1  # Don't use if possible, please create a new code and use it
     command_line = 2
     hostname_mismatch = 3
@@ -353,9 +355,16 @@ class ErrFilter(logging.Filter):
 def setup():
     u"""Initialize logging"""
     global _logger
+    global _log_timestamp
     if _logger:
         return
 
+    # have to get log options from commandline before initializing logger
+    # hackish? yes, but I see no other way other than external config
+    if u'--log-timestamp' in sys.argv[1:]:
+        _log_timestamp = True
+
+    # OK, now we can start setup
     _logger = logging.getLogger(u"duplicity")
 
     # Default verbosity allows notices and above
@@ -363,17 +372,64 @@ def setup():
 
     # stdout and stderr are for different logging levels
     outHandler = logging.StreamHandler(sys.stdout)
+    if _log_timestamp:
+        outHandler.setFormatter(DetailFormatter())
+    else:
+        outHandler.setFormatter(PrettyProgressFormatter())
     outHandler.addFilter(OutFilter())
     _logger.addHandler(outHandler)
 
     errHandler = logging.StreamHandler(sys.stderr)
+    if _log_timestamp:
+        errHandler.setFormatter(DetailFormatter())
+    else:
+        errHandler.setFormatter(PrettyProgressFormatter())
     errHandler.addFilter(ErrFilter())
     _logger.addHandler(errHandler)
 
 
+class PrettyProgressFormatter(logging.Formatter):
+    u"""Formatter that overwrites previous progress lines on ANSI terminals"""
+    last_record_was_progress = False
+
+    def __init__(self):
+        # 'message' will be appended by format()
+        # Note that we use our own, custom-created 'levelName' instead of the
+        # standard 'levelname'.  This is because the standard 'levelname' can
+        # be adjusted by any library anywhere in our stack without us knowing.
+        # But we control 'levelName'.
+        logging.Formatter.__init__(self, u"%(message)s")
+
+    def format(self, record):
+        s = logging.Formatter.format(self, record)
+
+        # So we don't overwrite actual log lines
+        if self.last_record_was_progress and record.transferProgress:
+            # Go up one line, then erase it
+            s = u"\033[F\033[2K" + s
+
+        self.last_record_was_progress = record.transferProgress
+
+        return s
+
+
+class DetailFormatter(logging.Formatter):
+    u"""Formatter that creates messages in a syntax somewhat like syslog."""
+    def __init__(self):
+        # 'message' will be appended by format()
+        # Note that we use our own, custom-created 'levelName' instead of the
+        # standard 'levelname'.  This is because the standard 'levelname' can
+        # be adjusted by any library anywhere in our stack without us knowing.
+        # But we control 'levelName'.
+        logging.Formatter.__init__(self, u"%(asctime)s %(levelName)s %(message)s")
+
+    def format(self, record):
+        s = logging.Formatter.format(self, record)
+        return s
+
+
 class MachineFormatter(logging.Formatter):
-    u"""Formatter that creates messages in a syntax easily consumable by other
-       processes."""
+    u"""Formatter that creates messages in a syntax easily consumable by other processes."""
     def __init__(self):
         # 'message' will be appended by format()
         # Note that we use our own, custom-created 'levelName' instead of the
@@ -433,4 +489,5 @@ def getverbosity():
 
 def shutdown():
     u"""Cleanup and flush loggers"""
+    global _logger
     logging.shutdown()
